from django.forms.formsets import formset_factory
from django.forms.models import inlineformset_factory, modelformset_factory
from django.http import HttpResponseRedirect
from django.views.generic.base import ContextMixin, TemplateResponseMixin, View
from django.views.generic.detail import (
    SingleObjectMixin,
    SingleObjectTemplateResponseMixin,
)
from django.views.generic.list import (
    MultipleObjectMixin,
    MultipleObjectTemplateResponseMixin,
)


class BaseFormSetFactory(object):
    """
    Base class for constructing a FormSet from `formset_factory` in a view.

    Calling `construct_formset` calls all other methods.
    """

    initial = []
    form_class = None
    formset_class = None
    prefix = None
    formset_kwargs = {}
    factory_kwargs = {}

    def construct_formset(self):
        """
        Returns an instance of the formset
        """
        formset_class = self.get_formset()
        if hasattr(self, "get_extra_form_kwargs"):
            klass = type(self).__name__
            raise DeprecationWarning(
                "Calling {0}.get_extra_form_kwargs is no longer supported. "
                "Set `form_kwargs` in {0}.formset_kwargs or override "
                "{0}.get_formset_kwargs() directly.".format(klass)
            )
        return formset_class(**self.get_formset_kwargs())

    def get_initial(self):
        """
        Returns a copy of the initial data to use for formsets on this view.
        """
        return self.initial[:]

    def get_prefix(self):
        """
        Returns the prefix used for formsets on this view.
        """
        return self.prefix

    def get_formset_class(self):
        """
        Returns the formset class to use in the formset factory
        """
        return self.formset_class

    def get_form_class(self):
        """
        Returns the form class to use with the formset in this view
        """
        return self.form_class

    def get_formset(self):
        """
        Returns the formset class from the formset factory
        """
        return formset_factory(self.get_form_class(), **self.get_factory_kwargs())

    def get_formset_kwargs(self):
        """
        Returns the keyword arguments for instantiating the formset.
        """
        kwargs = self.formset_kwargs.copy()
        kwargs.update({"initial": self.get_initial(), "prefix": self.get_prefix()})

        if self.request.method in ("POST", "PUT"):
            kwargs.update(
                {"data": self.request.POST.copy(), "files": self.request.FILES}
            )
        return kwargs

    def get_factory_kwargs(self):
        """
        Returns the keyword arguments for calling the formset factory
        """
        # Perform deprecation check
        for attr in [
            "extra",
            "max_num",
            "can_order",
            "can_delete",
            "ct_field",
            "formfield_callback",
            "fk_name",
            "widgets",
            "ct_fk_field",
        ]:
            if hasattr(self, attr):
                klass = type(self).__name__
                raise DeprecationWarning(
                    "Setting `{0}.{1}` at the class level is now deprecated. "
                    "Set `{0}.factory_kwargs` instead.".format(klass, attr)
                )

        kwargs = self.factory_kwargs.copy()
        if self.get_formset_class():
            kwargs["formset"] = self.get_formset_class()
        return kwargs


class BaseFormSetMixin(BaseFormSetFactory):
    def __init__(self, *args, **kwargs):
        from warnings import warn

        warn(
            "`extra_views.BaseFormSetMixin` has been renamed to "
            "`BaseFormSetFactory`. `BaseFormSetMixin` will be removed in "
            "a future release.",
            DeprecationWarning,
        )
        super(BaseFormSetMixin, self).__init__(*args, **kwargs)


class FormSetMixin(BaseFormSetFactory, ContextMixin):
    """
    A view mixin that provides a way to show and handle a single formset in a request.
    """

    success_url = None

    def get_success_url(self):
        """
        Returns the supplied URL.
        """
        if self.success_url:
            url = self.success_url
        else:
            # Default to returning to the same page
            url = self.request.get_full_path()
        return url

    def formset_valid(self, formset):
        """
        If the formset is valid redirect to the supplied URL
        """
        return HttpResponseRedirect(self.get_success_url())

    def formset_invalid(self, formset):
        """
        If the formset is invalid, re-render the context data with the
        data-filled formset and errors.
        """
        return self.render_to_response(self.get_context_data(formset=formset))


class ModelFormSetMixin(FormSetMixin, MultipleObjectMixin):
    """
    A view mixin that provides a way to show and handle a single model formset
    in a request.

    Uses `modelformset_factory`.
    """

    exclude = None
    fields = None

    def get_formset_kwargs(self):
        """
        Returns the keyword arguments for instantiating the formset.
        """
        kwargs = super(ModelFormSetMixin, self).get_formset_kwargs()
        kwargs["queryset"] = self.get_queryset()
        return kwargs

    def get_factory_kwargs(self):
        """
        Returns the keyword arguments for calling the formset factory
        """
        kwargs = super(ModelFormSetMixin, self).get_factory_kwargs()
        kwargs.setdefault("fields", self.fields)
        kwargs.setdefault("exclude", self.exclude)

        if self.get_form_class():
            kwargs["form"] = self.get_form_class()
        return kwargs

    def get_formset(self):
        """
        Returns the formset class from the model formset factory
        """
        return modelformset_factory(self.model, **self.get_factory_kwargs())

    def formset_valid(self, formset):
        """
        If the formset is valid, save the associated models.
        """
        self.object_list = formset.save()
        return super(ModelFormSetMixin, self).formset_valid(formset)


class BaseInlineFormSetFactory(BaseFormSetFactory):
    """
    Base class for constructing a FormSet from `inlineformset_factory` in a view.

    Calling `construct_formset` calls all other methods.
    """

    model = None
    inline_model = None
    exclude = None
    fields = None

    def get_inline_model(self):
        """
        Returns the inline model to use with the inline formset
        """
        return self.inline_model

    def get_formset_kwargs(self):
        """
        Returns the keyword arguments for instantiating the formset.
        """
        # Perform deprecation check
        if hasattr(self, "save_as_new"):
            klass = type(self).__name__
            raise DeprecationWarning(
                "Setting `{0}.save_as_new` at the class level is now "
                "deprecated. Set `{0}.formset_kwargs` instead.".format(klass)
            )
        kwargs = super(BaseInlineFormSetFactory, self).get_formset_kwargs()
        kwargs["instance"] = self.object
        return kwargs

    def get_factory_kwargs(self):
        """
        Returns the keyword arguments for calling the formset factory
        """
        kwargs = super(BaseInlineFormSetFactory, self).get_factory_kwargs()
        kwargs.setdefault("fields", self.fields)
        kwargs.setdefault("exclude", self.exclude)

        if self.get_form_class():
            kwargs["form"] = self.get_form_class()
        return kwargs

    def get_formset(self):
        """
        Returns the formset class from the inline formset factory
        """
        return inlineformset_factory(
            self.model, self.get_inline_model(), **self.get_factory_kwargs()
        )


class BaseInlineFormSetMixin(BaseInlineFormSetFactory):
    def __init__(self, *args, **kwargs):
        from warnings import warn

        warn(
            "`extra_views.BaseInlineFormSetMixin` has been renamed to "
            "`BaseInlineFormSetFactory`. `BaseInlineFormSetMixin` will be removed in "
            "a future release.",
            DeprecationWarning,
        )
        super(BaseInlineFormSetMixin, self).__init__(*args, **kwargs)


class InlineFormSetMixin(BaseInlineFormSetFactory, SingleObjectMixin, FormSetMixin):
    """
    A view mixin that provides a way to show and handle a single inline formset
    in a request.
    """

    def formset_valid(self, formset):
        self.object_list = formset.save()
        return super(InlineFormSetMixin, self).formset_valid(formset)


class ProcessFormSetView(View):
    """
    A mixin that processes a formset on POST.
    """

    def get(self, request, *args, **kwargs):
        """
        Handles GET requests and instantiates a blank version of the formset.
        """
        formset = self.construct_formset()
        return self.render_to_response(self.get_context_data(formset=formset))

    def post(self, request, *args, **kwargs):
        """
        Handles POST requests, instantiating a formset instance with the passed
        POST variables and then checked for validity.
        """
        formset = self.construct_formset()
        if formset.is_valid():
            return self.formset_valid(formset)
        else:
            return self.formset_invalid(formset)

    # PUT is a valid HTTP verb for creating (with a known URL) or editing an
    # object, note that browsers only support POST for now.
    def put(self, *args, **kwargs):
        return self.post(*args, **kwargs)


class BaseFormSetView(FormSetMixin, ProcessFormSetView):
    """
    A base view for displaying a formset
    """


class FormSetView(TemplateResponseMixin, BaseFormSetView):
    """
    A view for displaying a formset, and rendering a template response
    """


class BaseModelFormSetView(ModelFormSetMixin, ProcessFormSetView):
    """
    A base view for displaying a model formset
    """

    def get(self, request, *args, **kwargs):
        self.object_list = self.get_queryset()
        return super(BaseModelFormSetView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object_list = self.get_queryset()
        return super(BaseModelFormSetView, self).post(request, *args, **kwargs)


class ModelFormSetView(MultipleObjectTemplateResponseMixin, BaseModelFormSetView):
    """
    A view for displaying a model formset, and rendering a template response
    """


class BaseInlineFormSetView(InlineFormSetMixin, ProcessFormSetView):
    """
    A base view for displaying an inline formset for a queryset belonging to
    a parent model
    """

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(BaseInlineFormSetView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(BaseInlineFormSetView, self).post(request, *args, **kwargs)


class InlineFormSetView(SingleObjectTemplateResponseMixin, BaseInlineFormSetView):
    """
    A view for displaying an inline formset for a queryset belonging to a parent model
    """



from django.conf.urls import url
from django.views.generic import TemplateView

from .formsets import AddressFormSet
from .views import (
    AddressFormSetView,
    AddressFormSetViewKwargs,
    AddressFormSetViewNamed,
    EventCalendarView,
    FormAndFormSetOverrideView,
    ItemModelFormSetExcludeView,
    ItemModelFormSetView,
    OrderCreateNamedView,
    OrderCreateView,
    OrderItemFormSetView,
    OrderTagsView,
    OrderUpdateView,
    PagedModelFormSetView,
    SearchableItemListView,
    SortableItemListView,
)

urlpatterns = [
    url(r"^formset/simple/$", AddressFormSetView.as_view()),
    url(r"^formset/simple/named/$", AddressFormSetViewNamed.as_view()),
    url(r"^formset/simple/kwargs/$", AddressFormSetViewKwargs.as_view()),
    url(
        r"^formset/simple_redirect/$",
        AddressFormSetView.as_view(success_url="/formset/simple_redirect/valid/"),
    ),
    url(
        r"^formset/simple_redirect/valid/$",
        TemplateView.as_view(template_name="extra_views/success.html"),
    ),
    url(r"^formset/custom/$", AddressFormSetView.as_view(formset_class=AddressFormSet)),
    url(r"^modelformset/simple/$", ItemModelFormSetView.as_view()),
    url(r"^modelformset/exclude/$", ItemModelFormSetExcludeView.as_view()),
    url(r"^modelformset/custom/$", FormAndFormSetOverrideView.as_view()),
    url(r"^modelformset/paged/$", PagedModelFormSetView.as_view()),
    url(r"^inlineformset/(?P<pk>\d+)/$", OrderItemFormSetView.as_view()),
    url(r"^inlines/(\d+)/new/$", OrderCreateView.as_view()),
    url(r"^inlines/new/$", OrderCreateView.as_view()),
    url(r"^inlines/new/named/$", OrderCreateNamedView.as_view()),
    url(r"^inlines/(?P<pk>\d+)/$", OrderUpdateView.as_view()),
    url(r"^genericinlineformset/(?P<pk>\d+)/$", OrderTagsView.as_view()),
    url(r"^sortable/(?P<flag>\w+)/$", SortableItemListView.as_view()),
    url(r"^events/(?P<year>\d{4})/(?P<month>\w+)/$", EventCalendarView.as_view()),
    url(r"^searchable/$", SearchableItemListView.as_view()),
    url(
        r"^searchable/predefined_query/$",
        SearchableItemListView.as_view(define_query=True),
    ),
    url(r"^searchable/exact_query/$", SearchableItemListView.as_view(exact_query=True)),
    url(
        r"^searchable/wrong_lookup/$", SearchableItemListView.as_view(wrong_lookup=True)
    ),
]



# Generated by Django 2.1.13 on 2019-10-08 04:42

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [("contenttypes", "0002_remove_content_type_name")]

    operations = [
        migrations.CreateModel(
            name="Contact",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("email", models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("date", models.DateField()),
            ],
        ),
        migrations.CreateModel(
            name="Item",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("sku", models.CharField(max_length=13)),
                (
                    "price",
                    models.DecimalField(db_index=True, decimal_places=2, max_digits=12),
                ),
                (
                    "status",
                    models.SmallIntegerField(
                        choices=[
                            (0, "Placed"),
                            (1, "Charged"),
                            (2, "Shipped"),
                            (3, "Cancelled"),
                        ],
                        db_index=True,
                        default=0,
                    ),
                ),
                (
                    "date_placed",
                    models.DateField(
                        blank=True, default=django.utils.timezone.now, null=True
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Order",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("customer", models.CharField(blank=True, default="", max_length=255)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_modified", models.DateTimeField(auto_now=True)),
                ("action_on_save", models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("object_id", models.PositiveIntegerField(null=True)),
                (
                    "content_type",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="item",
            name="order",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="items",
                to="extra_views_tests.Order",
            ),
        ),
        migrations.AddField(
            model_name="contact",
            name="order",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="contacts",
                to="extra_views_tests.Order",
            ),
        ),
    ]



from __future__ import unicode_literals

import datetime
import math
from calendar import Calendar
from collections import defaultdict

from django.core.exceptions import ImproperlyConfigured
from django.db.models import Q
# Remove check when support for python < 3 is dropped.
# https://docs.djangoproject.com/en/3.0/releases/3.0/#features-deprecated-in-3-0
import sys
if sys.version_info[0] >= 3:
    from django.utils.translation import gettext_lazy as _
else:
    from django.utils.translation import ugettext_lazy as _
from django.views.generic.dates import (
    DateMixin,
    MonthMixin,
    YearMixin,
    _date_from_string,
)
from django.views.generic.list import BaseListView, MultipleObjectTemplateResponseMixin

DAYS = (
    _("Monday"),
    _("Tuesday"),
    _("Wednesday"),
    _("Thursday"),
    _("Friday"),
    _("Saturday"),
    _("Sunday"),
)


def daterange(start_date, end_date):
    """
    Returns an iterator of dates between two provided ones
    """
    for n in range(int((end_date - start_date).days + 1)):
        yield start_date + datetime.timedelta(n)


class BaseCalendarMonthView(DateMixin, YearMixin, MonthMixin, BaseListView):
    """
    A base view for displaying a calendar month
    """

    first_of_week = 0  # 0 = Monday, 6 = Sunday
    paginate_by = None  # We don't want to use this part of MultipleObjectMixin
    date_field = None
    end_date_field = None  # For supporting events with duration

    def get_paginate_by(self, queryset):
        if self.paginate_by is not None:
            raise ImproperlyConfigured(
                "'%s' cannot be paginated, it is a calendar view"
                % self.__class__.__name__
            )
        return None

    def get_allow_future(self):
        return True

    def get_end_date_field(self):
        """
        Returns the model field to use for end dates
        """
        return self.end_date_field

    def get_start_date(self, obj):
        """
        Returns the start date for a model instance
        """
        obj_date = getattr(obj, self.get_date_field())
        try:
            obj_date = obj_date.date()
        except AttributeError:
            # It's a date rather than datetime, so we use it as is
            pass
        return obj_date

    def get_end_date(self, obj):
        """
        Returns the end date for a model instance
        """
        obj_date = getattr(obj, self.get_end_date_field())
        try:
            obj_date = obj_date.date()
        except AttributeError:
            # It's a date rather than datetime, so we use it as is
            pass
        return obj_date

    def get_first_of_week(self):
        """
        Returns an integer representing the first day of the week.

        0 represents Monday, 6 represents Sunday.
        """
        if self.first_of_week is None:
            raise ImproperlyConfigured(
                "%s.first_of_week is required." % self.__class__.__name__
            )
        if self.first_of_week not in range(7):
            raise ImproperlyConfigured(
                "%s.first_of_week must be an integer between 0 and 6."
                % self.__class__.__name__
            )
        return self.first_of_week

    def get_queryset(self):
        """
        Returns a queryset of models for the month requested
        """
        qs = super(BaseCalendarMonthView, self).get_queryset()

        year = self.get_year()
        month = self.get_month()

        date_field = self.get_date_field()
        end_date_field = self.get_end_date_field()

        date = _date_from_string(
            year, self.get_year_format(), month, self.get_month_format()
        )

        since = date
        until = self.get_next_month(date)

        # Adjust our start and end dates to allow for next and previous
        # month edges
        if since.weekday() != self.get_first_of_week():
            diff = math.fabs(since.weekday() - self.get_first_of_week())
            since = since - datetime.timedelta(days=diff)

        if until.weekday() != ((self.get_first_of_week() + 6) % 7):
            diff = math.fabs(((self.get_first_of_week() + 6) % 7) - until.weekday())
            until = until + datetime.timedelta(days=diff)

        if end_date_field:
            # 5 possible conditions for showing an event:

            # 1) Single day event, starts after 'since'
            # 2) Multi-day event, starts after 'since' and ends before 'until'
            # 3) Starts before 'since' and ends after 'since' and before 'until'
            # 4) Starts after 'since' but before 'until' and ends after 'until'
            # 5) Starts before 'since' and ends after 'until'
            predicate1 = Q(**{"%s__gte" % date_field: since, end_date_field: None})
            predicate2 = Q(
                **{"%s__gte" % date_field: since, "%s__lt" % end_date_field: until}
            )
            predicate3 = Q(
                **{
                    "%s__lt" % date_field: since,
                    "%s__gte" % end_date_field: since,
                    "%s__lt" % end_date_field: until,
                }
            )
            predicate4 = Q(
                **{
                    "%s__gte" % date_field: since,
                    "%s__lt" % date_field: until,
                    "%s__gte" % end_date_field: until,
                }
            )
            predicate5 = Q(
                **{"%s__lt" % date_field: since, "%s__gte" % end_date_field: until}
            )
            return qs.filter(
                predicate1 | predicate2 | predicate3 | predicate4 | predicate5
            )
        return qs.filter(**{"%s__gte" % date_field: since})

    def get_context_data(self, **kwargs):
        """
        Injects variables necessary for rendering the calendar into the context.

        Variables added are: `calendar`, `weekdays`, `month`, `next_month` and
        `previous_month`.
        """
        data = super(BaseCalendarMonthView, self).get_context_data(**kwargs)

        year = self.get_year()
        month = self.get_month()

        date = _date_from_string(
            year, self.get_year_format(), month, self.get_month_format()
        )

        cal = Calendar(self.get_first_of_week())

        month_calendar = []
        now = datetime.datetime.utcnow()

        date_lists = defaultdict(list)
        multidate_objs = []

        for obj in data["object_list"]:
            obj_date = self.get_start_date(obj)
            end_date_field = self.get_end_date_field()

            if end_date_field:
                end_date = self.get_end_date(obj)
                if end_date and end_date != obj_date:
                    multidate_objs.append(
                        {
                            "obj": obj,
                            "range": [x for x in daterange(obj_date, end_date)],
                        }
                    )
                    continue  # We don't put multi-day events in date_lists
            date_lists[obj_date].append(obj)

        for week in cal.monthdatescalendar(date.year, date.month):
            week_range = set(daterange(week[0], week[6]))
            week_events = []

            for val in multidate_objs:
                intersect_length = len(week_range.intersection(val["range"]))

                if intersect_length:
                    # Event happens during this week
                    slot = 1
                    width = (
                        intersect_length
                    )  # How many days is the event during this week?
                    nowrap_previous = (
                        True
                    )  # Does the event continue from the previous week?
                    nowrap_next = True  # Does the event continue to the next week?

                    if val["range"][0] >= week[0]:
                        slot = 1 + (val["range"][0] - week[0]).days
                    else:
                        nowrap_previous = False
                    if val["range"][-1] > week[6]:
                        nowrap_next = False

                    week_events.append(
                        {
                            "event": val["obj"],
                            "slot": slot,
                            "width": width,
                            "nowrap_previous": nowrap_previous,
                            "nowrap_next": nowrap_next,
                        }
                    )

            week_calendar = {"events": week_events, "date_list": []}
            for day in week:
                week_calendar["date_list"].append(
                    {
                        "day": day,
                        "events": date_lists[day],
                        "today": day == now.date(),
                        "is_current_month": day.month == date.month,
                    }
                )
            month_calendar.append(week_calendar)

        data["calendar"] = month_calendar
        data["weekdays"] = [DAYS[x] for x in cal.iterweekdays()]
        data["month"] = date
        data["next_month"] = self.get_next_month(date)
        data["previous_month"] = self.get_previous_month(date)

        return data


class CalendarMonthView(MultipleObjectTemplateResponseMixin, BaseCalendarMonthView):
    """
    A view for displaying a calendar month, and rendering a template response
    """

    template_name_suffix = "_calendar_month"



from django import forms
from django.forms.formsets import BaseFormSet
from django.forms.models import BaseModelFormSet

COUNTRY_CHOICES = (
    ("gb", "Great Britain"),
    ("us", "United States"),
    ("ca", "Canada"),
    ("au", "Australia"),
    ("nz", "New Zealand"),
)


class AddressFormSet(BaseFormSet):
    def add_fields(self, form, index):
        super(AddressFormSet, self).add_fields(form, index)
        form.fields["county"] = forms.ChoiceField(choices=COUNTRY_CHOICES, initial="gb")


class BaseArticleFormSet(BaseModelFormSet):
    def add_fields(self, form, index):
        super(BaseArticleFormSet, self).add_fields(form, index)
        form.fields["notes"] = forms.CharField(initial="Write notes here")



from django.views import generic

from extra_views import (
    CalendarMonthView,
    CreateWithInlinesView,
    FormSetSuccessMessageMixin,
    FormSetView,
    InlineFormSetFactory,
    InlineFormSetView,
    ModelFormSetView,
    NamedFormsetsMixin,
    SearchableListMixin,
    SortableListMixin,
    SuccessMessageMixin,
    UpdateWithInlinesView,
)
from extra_views.generic import GenericInlineFormSetFactory, GenericInlineFormSetView

from .forms import AddressForm, ItemForm, OrderForm
from .formsets import BaseArticleFormSet
from .models import Event, Item, Order, Tag


class AddressFormSetView(FormSetSuccessMessageMixin, FormSetView):
    form_class = AddressForm
    template_name = "extra_views/address_formset.html"
    success_message = "Formset objects were created successfully!"


class AddressFormSetViewNamed(NamedFormsetsMixin, AddressFormSetView):
    inlines_names = ["AddressFormset"]


class AddressFormSetViewKwargs(FormSetView):
    # Used for testing class level kwargs
    form_class = AddressForm
    template_name = "extra_views/address_formset.html"
    formset_kwargs = {"auto_id": "id_test_%s", "form_kwargs": {"empty_permitted": True}}
    factory_kwargs = {"max_num": 27}
    prefix = "test_prefix"
    initial = [{"name": "address1"}]


class ItemModelFormSetView(ModelFormSetView):
    model = Item
    fields = ["name", "sku", "price", "order", "status"]
    template_name = "extra_views/item_formset.html"


class ItemModelFormSetExcludeView(ModelFormSetView):
    model = Item
    exclude = ["sku", "price"]
    template_name = "extra_views/item_formset.html"


class FormAndFormSetOverrideView(ModelFormSetView):
    model = Item
    form_class = ItemForm
    formset_class = BaseArticleFormSet
    template_name = "extra_views/item_formset.html"


class OrderItemFormSetView(InlineFormSetView):
    model = Order
    fields = ["name", "sku", "price", "order", "status"]
    inline_model = Item
    template_name = "extra_views/inline_formset.html"


class PagedModelFormSetView(ModelFormSetView):
    paginate_by = 2
    model = Item
    template_name = "extra_views/paged_formset.html"


class ItemsInline(InlineFormSetFactory):
    model = Item
    fields = ["name", "sku", "price", "order", "status"]


class TagsInline(GenericInlineFormSetFactory):
    model = Tag
    fields = ["name"]


class OrderCreateView(SuccessMessageMixin, CreateWithInlinesView):
    model = Order
    fields = ["name"]
    context_object_name = "order"
    inlines = [ItemsInline, TagsInline]
    template_name = "extra_views/order_and_items.html"
    success_message = "Order %(name)s was created successfully!"

    def form_valid(self, form):
        response = super(OrderCreateView, self).form_valid(form)
        # Update the response url to indicate that form_valid was called
        response["Location"] += "?form_valid_called=1"
        return response


class OrderCreateNamedView(NamedFormsetsMixin, OrderCreateView):
    inlines_names = ["Items", "Tags"]


class OrderUpdateView(UpdateWithInlinesView):
    model = Order
    form_class = OrderForm
    inlines = [ItemsInline, TagsInline]
    template_name = "extra_views/order_and_items.html"


class OrderTagsView(GenericInlineFormSetView):
    model = Order
    inline_model = Tag
    template_name = "extra_views/inline_formset.html"
    initial = [{"name": "test_tag_name"}]


class EventCalendarView(CalendarMonthView):
    template_name = "extra_views/event_calendar_month.html"
    model = Event
    month_format = "%b"
    date_field = "date"


class SearchableItemListView(SearchableListMixin, generic.ListView):
    template_name = "extra_views/item_list.html"
    search_fields = ["name", "sku"]
    search_date_fields = ["date_placed"]
    model = Item
    define_query = False
    exact_query = False
    wrong_lookup = False

    def get_search_query(self):
        if self.define_query:
            return "test B"
        else:
            return super(SearchableItemListView, self).get_search_query()

    def get(self, request, *args, **kwargs):
        if self.exact_query:
            self.search_fields = [("name", "iexact"), "sku"]
        elif self.wrong_lookup:
            self.search_fields = [("name", "gte"), "sku"]
        return super(SearchableItemListView, self).get(request, *args, **kwargs)


class SortableItemListView(SortableListMixin, generic.ListView):
    template_name = "extra_views/sortable_item_list.html"
    sort_fields = ["name", "sku"]
    model = Item

    def get(self, request, *args, **kwargs):
        if kwargs["flag"] == "fields_and_aliases":
            self.sort_fields_aliases = [("name", "by_name"), ("sku", "by_sku")]
        elif kwargs["flag"] == "aliases":
            self.sort_fields_aliases = [("name", "by_name"), ("sku", "by_sku")]
            self.sort_fields = []
        return super(SortableItemListView, self).get(request, *args, **kwargs)



from django.contrib import messages
from django.forms.formsets import all_valid
from django.views.generic.base import ContextMixin
from django.views.generic.detail import SingleObjectTemplateResponseMixin
from django.views.generic.edit import FormView, ModelFormMixin

from extra_views.formsets import BaseInlineFormSetFactory


class InlineFormSetFactory(BaseInlineFormSetFactory):
    """
    Class used to create an `InlineFormSet` from `inlineformset_factory` as
    one of multiple `InlineFormSet`s within a single view.

    Subclasses `BaseInlineFormSetFactory` and passes in the necessary view arguments.
    """

    def __init__(self, parent_model, request, instance, view_kwargs=None, view=None):
        self.inline_model = self.model
        self.model = parent_model
        self.request = request
        self.object = instance
        self.kwargs = view_kwargs
        self.view = view

    def construct_formset(self):
        """
        Overrides construct_formset to attach the model class as
        an attribute of the returned formset instance.
        """
        formset = super(InlineFormSetFactory, self).construct_formset()
        formset.model = self.inline_model
        return formset


class InlineFormSet(InlineFormSetFactory):
    def __init__(self, *args, **kwargs):
        from warnings import warn

        warn(
            "`extra_views.InlineFormSet` has been renamed to `InlineFormSetFactory`. "
            "`InlineFormSet` will be removed in a future release.",
            DeprecationWarning,
        )
        super(InlineFormSet, self).__init__(*args, **kwargs)


class ModelFormWithInlinesMixin(ModelFormMixin):
    """
    A mixin that provides a way to show and handle a modelform and inline
    formsets in a request.

    The inlines should be subclasses of `InlineFormSetFactory`.
    """

    inlines = []

    def get_inlines(self):
        """
        Returns the inline formset classes
        """
        return self.inlines

    def forms_valid(self, form, inlines):
        """
        If the form and formsets are valid, save the associated models.
        """
        response = self.form_valid(form)
        for formset in inlines:
            formset.save()
        return response

    def forms_invalid(self, form, inlines):
        """
        If the form or formsets are invalid, re-render the context data with the
        data-filled form and formsets and errors.
        """
        return self.render_to_response(
            self.get_context_data(form=form, inlines=inlines)
        )

    def construct_inlines(self):
        """
        Returns the inline formset instances
        """
        inline_formsets = []
        for inline_class in self.get_inlines():
            inline_instance = inline_class(
                self.model, self.request, self.object, self.kwargs, self
            )
            inline_formset = inline_instance.construct_formset()
            inline_formsets.append(inline_formset)
        return inline_formsets


class ProcessFormWithInlinesView(FormView):
    """
    A mixin that renders a form and inline formsets on GET and processes it on POST.
    """

    def get(self, request, *args, **kwargs):
        """
        Handles GET requests and instantiates a blank version of the form and formsets.
        """
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        inlines = self.construct_inlines()
        return self.render_to_response(
            self.get_context_data(form=form, inlines=inlines, **kwargs)
        )

    def post(self, request, *args, **kwargs):
        """
        Handles POST requests, instantiating a form and formset instances with the
        passed POST variables and then checked for validity.
        """
        form_class = self.get_form_class()
        form = self.get_form(form_class)

        initial_object = self.object
        if form.is_valid():
            self.object = form.save(commit=False)
            form_validated = True
        else:
            form_validated = False

        inlines = self.construct_inlines()

        if all_valid(inlines) and form_validated:
            return self.forms_valid(form, inlines)
        self.object = initial_object
        return self.forms_invalid(form, inlines)

    # PUT is a valid HTTP verb for creating (with a known URL) or editing an
    # object, note that browsers only support POST for now.
    def put(self, *args, **kwargs):
        return self.post(*args, **kwargs)


class BaseCreateWithInlinesView(ModelFormWithInlinesMixin, ProcessFormWithInlinesView):
    """
    Base view for creating an new object instance with related model instances.

    Using this base class requires subclassing to provide a response mixin.
    """

    def get(self, request, *args, **kwargs):
        self.object = None
        return super(BaseCreateWithInlinesView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = None
        return super(BaseCreateWithInlinesView, self).post(request, *args, **kwargs)


class CreateWithInlinesView(
    SingleObjectTemplateResponseMixin, BaseCreateWithInlinesView
):
    """
    View for creating a new object instance with related model instances,
    with a response rendered by template.
    """

    template_name_suffix = "_form"


class BaseUpdateWithInlinesView(ModelFormWithInlinesMixin, ProcessFormWithInlinesView):
    """
    Base view for updating an existing object with related model instances.

    Using this base class requires subclassing to provide a response mixin.
    """

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(BaseUpdateWithInlinesView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(BaseUpdateWithInlinesView, self).post(request, *args, **kwargs)


class UpdateWithInlinesView(
    SingleObjectTemplateResponseMixin, BaseUpdateWithInlinesView
):
    """
    View for updating an object with related model instances,
    with a response rendered by template.
    """

    template_name_suffix = "_form"


class NamedFormsetsMixin(ContextMixin):
    """
    A mixin for use with `CreateWithInlinesView` or `UpdateWithInlinesView` that lets
    you define the context variable for each inline.
    """

    inlines_names = []

    def get_inlines_names(self):
        """
        Returns a list of names of context variables for each inline in `inlines`.
        """
        return self.inlines_names

    def get_context_data(self, **kwargs):
        """
        If `inlines_names` has been defined, add each formset to the context under
        its corresponding entry in `inlines_names`
        """
        context = {}
        inlines_names = self.get_inlines_names()

        if inlines_names:
            # We have formset or inlines in context, but never both
            context.update(zip(inlines_names, kwargs.get("inlines", [])))
            if "formset" in kwargs:
                context[inlines_names[0]] = kwargs["formset"]
        context.update(kwargs)
        return super(NamedFormsetsMixin, self).get_context_data(**context)


class SuccessMessageMixin(object):
    """
    Adds success message on views with inlines if django.contrib.messages framework
    is used.
    In order to use just add mixin in to inheritance before main class, e.g.:
    class MyCreateWithInlinesView (SuccessMessageMixin, CreateWithInlinesView):
        success_message='Something was created!'
    """

    success_message = ""

    def forms_valid(self, form, inlines):
        response = super(SuccessMessageMixin, self).forms_valid(form, inlines)
        success_message = self.get_success_message(form.cleaned_data, inlines)
        if success_message:
            messages.success(self.request, success_message)
        return response

    def get_success_message(self, cleaned_data, inlines):
        return self.success_message % cleaned_data


class FormSetSuccessMessageMixin(object):
    """
    Adds success message on FormSet views if django.contrib.messages framework
    is used. In order to use just add mixin in to inheritance before main
    class, e.g.:
    class MyFormSetView (FormSetSuccessMessageMixin, ModelFormSetView):
        success_message='Something was created!'
    """

    success_message = ""

    def formset_valid(self, formset):
        response = super(FormSetSuccessMessageMixin, self).formset_valid(formset)
        success_message = self.get_success_message(formset)
        if success_message:
            messages.success(self.request, success_message)
        return response

    def get_success_message(self, formset):
        return self.success_message



import os

PROJECT_DIR = os.path.dirname(os.path.abspath(__file__))
BASE_DIR = os.path.dirname(PROJECT_DIR)

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

TEMPLATES = [
    {"BACKEND": "django.template.backends.django.DjangoTemplates", "APP_DIRS": True}
]

INSTALLED_APPS = [
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "extra_views",
    "extra_views_tests",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "extra_views_tests.urls"

SECRET_KEY = "something not very secret"



from django.contrib.contenttypes.forms import generic_inlineformset_factory

from extra_views.formsets import (
    BaseInlineFormSetFactory,
    BaseInlineFormSetView,
    InlineFormSetMixin,
    InlineFormSetView,
)


class BaseGenericInlineFormSetFactory(BaseInlineFormSetFactory):
    """
    Base class for constructing a GenericInlineFormSet from
    `generic_inlineformset_factory` in a view.
    """

    def get_formset(self):
        """
        Returns the final formset class from generic_inlineformset_factory.
        """
        result = generic_inlineformset_factory(
            self.inline_model, **self.get_factory_kwargs()
        )
        return result


class BaseGenericInlineFormSetMixin(BaseGenericInlineFormSetFactory):
    def __init__(self, *args, **kwargs):
        from warnings import warn

        warn(
            "`extra_views.BaseGenericInlineFormSetMixin` has been renamed to "
            "`BaseGenericInlineFormSetFactory`. `BaseGenericInlineFormSetMixin` "
            "will be removed in a future release.",
            DeprecationWarning,
        )
        super(BaseGenericInlineFormSetMixin, self).__init__(*args, **kwargs)


class GenericInlineFormSetFactory(BaseGenericInlineFormSetFactory):
    """
    Class used to create a `GenericInlineFormSet` from `generic_inlineformset_factory`
    as one of multiple `GenericInlineFormSet`s within a single view.

    Subclasses `BaseGenericInlineFormSetFactory` and passes in the necessary view
    arguments.
    """

    def __init__(self, parent_model, request, instance, view_kwargs=None, view=None):
        self.inline_model = self.model
        self.model = parent_model
        self.request = request
        self.object = instance
        self.kwargs = view_kwargs
        self.view = view


class GenericInlineFormSet(GenericInlineFormSetFactory):
    def __init__(self, *args, **kwargs):
        from warnings import warn

        warn(
            "`extra_views.GenericInlineFormSet` has been renamed to "
            "`GenericInlineFormSetFactory`. `GenericInlineFormSet` "
            "will be removed in a future release.",
            DeprecationWarning,
        )
        super(GenericInlineFormSet, self).__init__(*args, **kwargs)


class GenericInlineFormSetMixin(BaseGenericInlineFormSetFactory, InlineFormSetMixin):
    """
    A mixin that provides a way to show and handle a generic inline formset in a
    request.
    """


class BaseGenericInlineFormSetView(GenericInlineFormSetMixin, BaseInlineFormSetView):
    """
    A base view for displaying a generic inline formset
    """


class GenericInlineFormSetView(BaseGenericInlineFormSetView, InlineFormSetView):
    """
    A view for displaying a generic inline formset for a queryset belonging to a
    parent model
    """



import datetime

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models

try:
    from django.utils.timezone import now
except ImportError:
    now = datetime.datetime.now


STATUS_CHOICES = ((0, "Placed"), (1, "Charged"), (2, "Shipped"), (3, "Cancelled"))


class Order(models.Model):
    name = models.CharField(max_length=255)
    customer = models.CharField(max_length=255, default="", blank=True)
    date_created = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)
    action_on_save = models.BooleanField(default=False)

    def get_absolute_url(self):
        return "/inlines/%i/" % self.pk

    def __str__(self):
        return self.name


class Item(models.Model):
    name = models.CharField(max_length=255)
    sku = models.CharField(max_length=13)
    price = models.DecimalField(decimal_places=2, max_digits=12, db_index=True)
    order = models.ForeignKey(Order, related_name="items", on_delete=models.CASCADE)
    status = models.SmallIntegerField(default=0, choices=STATUS_CHOICES, db_index=True)
    date_placed = models.DateField(default=now, null=True, blank=True)

    def __str__(self):
        return "%s (%s)" % (self.name, self.sku)


class Contact(models.Model):
    name = models.CharField(max_length=255)
    email = models.CharField(max_length=255)
    order = models.ForeignKey(Order, related_name="contacts", on_delete=models.CASCADE)

    def __str__(self):
        return self.name


class Tag(models.Model):
    name = models.CharField(max_length=255)
    content_type = models.ForeignKey(ContentType, null=True, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField(null=True)
    content_object = GenericForeignKey("content_type", "object_id")

    def __str__(self):
        return self.name


class Event(models.Model):
    name = models.CharField(max_length=255)
    date = models.DateField()

    def __str__(self):
        return self.name



from __future__ import unicode_literals

import datetime
from decimal import Decimal as D
from unittest import expectedFailure

from django.contrib.messages import get_messages
from django.core.exceptions import ImproperlyConfigured
from django.forms import ValidationError
from django.test import TestCase

from .models import Event, Item, Order, Tag


class FormSetViewTests(TestCase):
    management_data = {
        "form-TOTAL_FORMS": "2",
        "form-INITIAL_FORMS": "0",
        "form-MAX_NUM_FORMS": "",
    }

    def test_create(self):
        res = self.client.get("/formset/simple/")
        self.assertEqual(res.status_code, 200)
        self.assertTrue("formset" in res.context)
        self.assertFalse("form" in res.context)
        self.assertTemplateUsed(res, "extra_views/address_formset.html")
        self.assertEqual(
            res.context["formset"].__class__.__name__, "AddressFormFormSet"
        )

    def test_formset_named(self):
        res = self.client.get("/formset/simple/named/")
        self.assertEqual(res.status_code, 200)
        self.assertEqual(res.context["formset"], res.context["AddressFormset"])

    def test_missing_management(self):
        with self.assertRaises(ValidationError):
            self.client.post("/formset/simple/", {})

    def test_success(self):
        res = self.client.post("/formset/simple/", self.management_data, follow=True)
        self.assertRedirects(res, "/formset/simple/", status_code=302)

    def test_success_message(self):
        res = self.client.post("/formset/simple/", self.management_data, follow=True)
        messages = [
            message.__str__() for message in get_messages(res.context["view"].request)
        ]
        self.assertIn("Formset objects were created successfully!", messages)

    @expectedFailure
    def test_put(self):
        res = self.client.put("/formset/simple/", self.management_data, follow=True)
        self.assertRedirects(res, "/formset/simple/", status_code=302)

    def test_success_url(self):
        res = self.client.post(
            "/formset/simple_redirect/", self.management_data, follow=True
        )
        self.assertRedirects(res, "/formset/simple_redirect/valid/")

    def test_invalid(self):
        data = {
            "form-0-name": "Joe Bloggs",
            "form-0-city": "",
            "form-0-line1": "",
            "form-0-line2": "",
            "form-0-postcode": "",
        }
        data.update(self.management_data)

        res = self.client.post("/formset/simple/", data, follow=True)
        self.assertEqual(res.status_code, 200)
        self.assertTrue("postcode" in res.context["formset"].errors[0])

    def test_formset_class(self):
        res = self.client.get("/formset/custom/")
        self.assertEqual(res.status_code, 200)

    def test_inital(self):
        res = self.client.get("/formset/simple/kwargs/")
        self.assertEqual(res.status_code, 200)
        initial_forms = res.context["formset"].initial_forms
        self.assertTrue(initial_forms)
        self.assertEqual(initial_forms[0].initial, {"name": "address1"})

    def test_prefix(self):
        res = self.client.get("/formset/simple/kwargs/")
        self.assertEqual(res.status_code, 200)
        self.assertEqual(res.context["formset"].management_form.prefix, "test_prefix")

    def test_factory_kwargs(self):
        res = self.client.get("/formset/simple/kwargs/")
        self.assertEqual(res.status_code, 200)
        self.assertEqual(
            res.context["formset"].management_form.initial["MAX_NUM_FORMS"], 27
        )

    def test_formset_kwargs(self):
        res = self.client.get("/formset/simple/kwargs/")
        self.assertEqual(res.status_code, 200)
        self.assertEqual(res.context["formset"].management_form.auto_id, "id_test_%s")
        initial_forms = res.context["formset"].initial_forms
        self.assertTrue(initial_forms)
        self.assertTrue(initial_forms[0].empty_permitted)


class ModelFormSetViewTests(TestCase):
    management_data = {
        "form-TOTAL_FORMS": "2",
        "form-INITIAL_FORMS": "0",
        "form-MAX_NUM_FORMS": "",
    }

    def test_create(self):
        res = self.client.get("/modelformset/simple/")
        self.assertEqual(res.status_code, 200)
        self.assertTrue("formset" in res.context)
        self.assertFalse("form" in res.context)
        self.assertTemplateUsed(res, "extra_views/item_formset.html")
        self.assertEqual(res.context["formset"].__class__.__name__, "ItemFormFormSet")

    def test_override(self):
        res = self.client.get("/modelformset/custom/")
        self.assertEqual(res.status_code, 200)
        form = res.context["formset"].forms[0]
        self.assertEqual(form["flag"].value(), True)
        self.assertEqual(form["notes"].value(), "Write notes here")

    def test_post(self):
        order = Order(name="Dummy Order")
        order.save()

        data = {
            "form-0-name": "Bubble Bath",
            "form-0-sku": "1234567890123",
            "form-0-price": D("9.99"),
            "form-0-order": order.id,
            "form-0-status": 0,
        }
        data.update(self.management_data)
        data["form-TOTAL_FORMS"] = "1"
        res = self.client.post("/modelformset/simple/", data, follow=True)
        self.assertEqual(res.status_code, 200)
        self.assertEqual(Item.objects.all().count(), 1)

    def test_context(self):
        order = Order(name="Dummy Order")
        order.save()

        for i in range(10):
            item = Item(
                name="Item %i" % i,
                sku=str(i) * 13,
                price=D("9.99"),
                order=order,
                status=0,
            )
            item.save()

        res = self.client.get("/modelformset/simple/")
        self.assertTrue("object_list" in res.context)
        self.assertEqual(len(res.context["object_list"]), 10)

    def test_fields_is_used(self):
        res = self.client.get("/modelformset/simple/")
        self.assertEqual(res.status_code, 200)
        fields = res.context["formset"].empty_form.fields
        self.assertIn("sku", fields)
        self.assertNotIn("date_placed", fields)

    def test_exclude_is_used(self):
        res = self.client.get("/modelformset/exclude/")
        self.assertEqual(res.status_code, 200)
        fields = res.context["formset"].empty_form.fields
        self.assertIn("date_placed", fields)
        self.assertNotIn("sku", fields)


class InlineFormSetViewTests(TestCase):
    management_data = {
        "items-TOTAL_FORMS": "2",
        "items-INITIAL_FORMS": "0",
        "items-MAX_NUM_FORMS": "",
    }

    def test_create(self):
        order = Order(name="Dummy Order")
        order.save()

        for i in range(10):
            item = Item(
                name="Item %i" % i,
                sku=str(i) * 13,
                price=D("9.99"),
                order=order,
                status=0,
            )
            item.save()

        res = self.client.get("/inlineformset/{}/".format(order.id))

        self.assertTrue("object" in res.context)
        self.assertTrue("order" in res.context)

        self.assertEqual(res.status_code, 200)
        self.assertTrue("formset" in res.context)
        self.assertFalse("form" in res.context)

    def test_post(self):
        order = Order(name="Dummy Order")
        order.save()
        data = {}
        data.update(self.management_data)

        res = self.client.post("/inlineformset/{}/".format(order.id), data, follow=True)
        self.assertEqual(res.status_code, 200)
        self.assertTrue("formset" in res.context)
        self.assertFalse("form" in res.context)

    def test_save(self):
        order = Order(name="Dummy Order")
        order.save()
        data = {
            "items-0-name": "Bubble Bath",
            "items-0-sku": "1234567890123",
            "items-0-price": D("9.99"),
            "items-0-status": 0,
            "items-1-DELETE": True,
        }
        data.update(self.management_data)

        self.assertEqual(0, order.items.count())
        res = self.client.post("/inlineformset/{}/".format(order.id), data, follow=True)
        order = Order.objects.get(id=order.id)

        context_instance = res.context["formset"][0].instance

        self.assertEqual("Bubble Bath", context_instance.name)
        self.assertEqual("", res.context["formset"][1].instance.name)

        self.assertEqual(1, order.items.count())


class GenericInlineFormSetViewTests(TestCase):
    def test_get(self):
        order = Order(name="Dummy Order")
        order.save()

        order2 = Order(name="Other Order")
        order2.save()

        tag = Tag(name="Test", content_object=order)
        tag.save()

        tag = Tag(name="Test2", content_object=order2)
        tag.save()

        res = self.client.get("/genericinlineformset/{}/".format(order.id))

        self.assertEqual(res.status_code, 200)
        self.assertTrue("formset" in res.context)
        self.assertFalse("form" in res.context)
        self.assertEqual("Test", res.context["formset"].forms[0]["name"].value())

    def test_post(self):
        order = Order(name="Dummy Order")
        order.save()

        tag = Tag(name="Test", content_object=order)
        tag.save()

        data = {
            "extra_views_tests-tag-content_type-object_id-TOTAL_FORMS": 3,
            "extra_views_tests-tag-content_type-object_id-INITIAL_FORMS": 1,
            "extra_views_tests-tag-content_type-object_id-MAX_NUM_FORMS": "",
            "extra_views_tests-tag-content_type-object_id-0-name": "Updated",
            "extra_views_tests-tag-content_type-object_id-0-id": 1,
            "extra_views_tests-tag-content_type-object_id-1-DELETE": True,
            "extra_views_tests-tag-content_type-object_id-2-DELETE": True,
        }

        res = self.client.post(
            "/genericinlineformset/{}/".format(order.id), data, follow=True
        )
        self.assertEqual(res.status_code, 200)
        self.assertEqual("Updated", res.context["formset"].forms[0]["name"].value())
        self.assertEqual(1, Tag.objects.count())

    def test_post2(self):
        order = Order(name="Dummy Order")
        order.save()

        tag = Tag(name="Test", content_object=order)
        tag.save()

        data = {
            "extra_views_tests-tag-content_type-object_id-TOTAL_FORMS": 3,
            "extra_views_tests-tag-content_type-object_id-INITIAL_FORMS": 1,
            "extra_views_tests-tag-content_type-object_id-MAX_NUM_FORMS": "",
            "extra_views_tests-tag-content_type-object_id-0-name": "Updated",
            "extra_views_tests-tag-content_type-object_id-0-id": tag.id,
            "extra_views_tests-tag-content_type-object_id-1-name": "Tag 2",
            "extra_views_tests-tag-content_type-object_id-2-name": "Tag 3",
        }

        res = self.client.post(
            "/genericinlineformset/{}/".format(order.id), data, follow=True
        )
        self.assertEqual(res.status_code, 200)
        self.assertEqual(3, Tag.objects.count())

    def test_intial_data_is_used(self):
        # Specific test for initial data in genericinlineformset
        order = Order(name="Dummy Order")
        order.save()
        res = self.client.get("/genericinlineformset/{}/".format(order.id))
        self.assertEqual(res.status_code, 200)
        extra_forms = res.context["formset"].extra_forms
        self.assertTrue(extra_forms)
        self.assertEqual(extra_forms[0].initial, {"name": "test_tag_name"})


class ModelWithInlinesTests(TestCase):
    def test_create(self):
        res = self.client.get("/inlines/new/")
        self.assertEqual(res.status_code, 200)
        self.assertEqual(0, Tag.objects.count())

        data = {
            "name": "Dummy Order",
            "items-TOTAL_FORMS": "2",
            "items-INITIAL_FORMS": "0",
            "items-MAX_NUM_FORMS": "",
            "items-0-name": "Bubble Bath",
            "items-0-sku": "1234567890123",
            "items-0-price": D("9.99"),
            "items-0-status": 0,
            "items-1-DELETE": True,
            "extra_views_tests-tag-content_type-object_id-TOTAL_FORMS": 2,
            "extra_views_tests-tag-content_type-object_id-INITIAL_FORMS": 0,
            "extra_views_tests-tag-content_type-object_id-MAX_NUM_FORMS": "",
            "extra_views_tests-tag-content_type-object_id-0-name": "Test",
            "extra_views_tests-tag-content_type-object_id-1-DELETE": True,
        }

        res = self.client.post("/inlines/new/", data, follow=True)

        self.assertTrue("object" in res.context)
        self.assertTrue("order" in res.context)

        self.assertEqual(res.status_code, 200)
        self.assertEqual(1, Tag.objects.count())

        # Check that form_valid has been called.
        self.assertRedirects(res, "/inlines/1/?form_valid_called=1")

    def test_create_success_message(self):
        res = self.client.get("/inlines/new/")
        self.assertEqual(res.status_code, 200)
        self.assertEqual(0, Tag.objects.count())

        data = {
            "name": "Dummy Order",
            "items-TOTAL_FORMS": "2",
            "items-INITIAL_FORMS": "0",
            "items-MAX_NUM_FORMS": "",
            "items-0-name": "Bubble Bath",
            "items-0-sku": "1234567890123",
            "items-0-price": D("9.99"),
            "items-0-status": 0,
            "items-1-DELETE": True,
            "extra_views_tests-tag-content_type-object_id-TOTAL_FORMS": 2,
            "extra_views_tests-tag-content_type-object_id-INITIAL_FORMS": 0,
            "extra_views_tests-tag-content_type-object_id-MAX_NUM_FORMS": "",
            "extra_views_tests-tag-content_type-object_id-0-name": "Test",
            "extra_views_tests-tag-content_type-object_id-1-DELETE": True,
        }

        res = self.client.post("/inlines/new/", data, follow=True)

        messages = [
            message.__str__() for message in get_messages(res.context["view"].request)
        ]
        self.assertIn("Order Dummy Order was created successfully!", messages)

    def test_named_create(self):
        res = self.client.get("/inlines/new/named/")
        self.assertEqual(res.status_code, 200)
        self.assertEqual(res.context["Items"], res.context["inlines"][0])
        self.assertEqual(res.context["Tags"], res.context["inlines"][1])

    def test_validation(self):
        data = {
            "items-TOTAL_FORMS": "2",
            "items-INITIAL_FORMS": "0",
            "items-MAX_NUM_FORMS": "",
            "items-0-name": "Test Item 1",
            "items-0-sku": "",
            "items-0-price": "",
            "items-0-status": 0,
            "items-1-name": "",
            "items-1-sku": "",
            "items-1-price": "",
            "items-1-status": "",
            "items-1-DELETE": True,
            "extra_views_tests-tag-content_type-object_id-TOTAL_FORMS": 2,
            "extra_views_tests-tag-content_type-object_id-INITIAL_FORMS": 0,
            "extra_views_tests-tag-content_type-object_id-MAX_NUM_FORMS": "",
            "extra_views_tests-tag-content_type-object_id-0-name": "Test",
            "extra_views_tests-tag-content_type-object_id-1-DELETE": True,
        }

        res = self.client.post("/inlines/new/", data, follow=True)
        self.assertEqual(len(res.context["form"].errors), 1)
        self.assertEqual(len(res.context["inlines"][0].errors[0]), 2)

    def test_view_object_is_none_after_failed_validation_for_createview(self):
        # We are testing that view.object = None even if the form validates
        # but one of the inline formsets does not.
        data = {
            "name": "Dummy Order",
            "items-TOTAL_FORMS": "2",
            "items-INITIAL_FORMS": "0",
            "items-MAX_NUM_FORMS": "",
            "items-0-name": "Test Item 1",
            "items-0-sku": "",
            "items-0-price": "",
            "items-0-status": 0,
            "items-1-name": "",
            "items-1-sku": "",
            "items-1-price": "",
            "items-1-status": "",
            "items-1-DELETE": True,
            "extra_views_tests-tag-content_type-object_id-TOTAL_FORMS": 2,
            "extra_views_tests-tag-content_type-object_id-INITIAL_FORMS": 0,
            "extra_views_tests-tag-content_type-object_id-MAX_NUM_FORMS": "",
            "extra_views_tests-tag-content_type-object_id-0-name": "Test",
            "extra_views_tests-tag-content_type-object_id-1-DELETE": True,
        }

        res = self.client.post("/inlines/new/", data, follow=True)
        self.assertEqual(len(res.context["form"].errors), 0)
        self.assertEqual(len(res.context["inlines"][0].errors[0]), 2)
        self.assertEqual(res.context["view"].object, None)

    def test_update(self):
        order = Order(name="Dummy Order")
        order.save()

        item_ids = []
        for i in range(2):
            item = Item(
                name="Item %i" % i,
                sku=str(i) * 13,
                price=D("9.99"),
                order=order,
                status=0,
            )
            item.save()
            item_ids.append(item.id)

        tag = Tag(name="Test", content_object=order)
        tag.save()

        res = self.client.get("/inlines/{}/".format(order.id))

        self.assertEqual(res.status_code, 200)
        order = Order.objects.get(id=order.id)

        self.assertEqual(2, order.items.count())
        self.assertEqual("Item 0", order.items.all()[0].name)

        data = {
            "name": "Dummy Order",
            "items-TOTAL_FORMS": "4",
            "items-INITIAL_FORMS": "2",
            "items-MAX_NUM_FORMS": "",
            "items-0-name": "Bubble Bath",
            "items-0-sku": "1234567890123",
            "items-0-price": D("9.99"),
            "items-0-status": 0,
            "items-0-id": item_ids[0],
            "items-1-name": "Bubble Bath",
            "items-1-sku": "1234567890123",
            "items-1-price": D("9.99"),
            "items-1-status": 0,
            "items-1-id": item_ids[1],
            "items-2-name": "Bubble Bath",
            "items-2-sku": "1234567890123",
            "items-2-price": D("9.99"),
            "items-2-status": 0,
            "items-3-DELETE": True,
            "extra_views_tests-tag-content_type-object_id-TOTAL_FORMS": 3,
            "extra_views_tests-tag-content_type-object_id-INITIAL_FORMS": 1,
            "extra_views_tests-tag-content_type-object_id-MAX_NUM_FORMS": "",
            "extra_views_tests-tag-content_type-object_id-0-name": "Test",
            "extra_views_tests-tag-content_type-object_id-0-id": tag.id,
            "extra_views_tests-tag-content_type-object_id-0-DELETE": True,
            "extra_views_tests-tag-content_type-object_id-1-name": "Test 2",
            "extra_views_tests-tag-content_type-object_id-2-name": "Test 3",
        }

        res = self.client.post("/inlines/{}/".format(order.id), data)
        self.assertEqual(res.status_code, 302)
        # Test that the returned url is the same as the instances absolute url.
        self.assertEqual(res.url, order.get_absolute_url())

        order = Order.objects.get(id=order.id)

        self.assertEqual(3, order.items.count())
        self.assertEqual(2, Tag.objects.count())
        self.assertEqual("Bubble Bath", order.items.all()[0].name)

    def test_parent_instance_saved_in_form_save(self):
        order = Order(name="Dummy Order")
        order.save()

        data = {
            "name": "Dummy Order",
            "items-TOTAL_FORMS": "0",
            "items-INITIAL_FORMS": "0",
            "items-MAX_NUM_FORMS": "",
            "extra_views_tests-tag-content_type-object_id-TOTAL_FORMS": "0",
            "extra_views_tests-tag-content_type-object_id-INITIAL_FORMS": "0",
            "extra_views_tests-tag-content_type-object_id-MAX_NUM_FORMS": "",
        }

        res = self.client.post("/inlines/{}/".format(order.id), data)
        self.assertEqual(res.status_code, 302)

        order = Order.objects.get(id=order.id)
        self.assertTrue(order.action_on_save)

    def test_url_arg(self):
        """
        Regression test for #122: get_context_data should not be called with *args
        """
        res = self.client.get("/inlines/123/new/")
        self.assertEqual(res.status_code, 200)


class CalendarViewTests(TestCase):
    def test_create(self):
        event = Event(name="Test Event", date=datetime.date(2012, 1, 1))
        event.save()

        res = self.client.get("/events/2012/jan/")
        self.assertEqual(res.status_code, 200)


class SearchableListTests(TestCase):
    def setUp(self):
        order = Order(name="Dummy Order")
        order.save()
        Item.objects.create(
            sku="1A",
            name="test A",
            order=order,
            price=0,
            date_placed=datetime.date(2012, 1, 1),
        )
        Item.objects.create(
            sku="1B",
            name="test B",
            order=order,
            price=0,
            date_placed=datetime.date(2012, 2, 1),
        )
        Item.objects.create(
            sku="C",
            name="test",
            order=order,
            price=0,
            date_placed=datetime.date(2012, 3, 1),
        )

    def test_search(self):
        res = self.client.get("/searchable/", data={"q": "1A test"})
        self.assertEqual(res.status_code, 200)
        self.assertEqual(1, len(res.context["object_list"]))

        res = self.client.get("/searchable/", data={"q": "1Atest"})
        self.assertEqual(res.status_code, 200)
        self.assertEqual(0, len(res.context["object_list"]))

        # date search
        res = self.client.get("/searchable/", data={"q": "01.01.2012"})
        self.assertEqual(res.status_code, 200)
        self.assertEqual(1, len(res.context["object_list"]))

        res = self.client.get("/searchable/", data={"q": "02.01.2012"})
        self.assertEqual(res.status_code, 200)
        self.assertEqual(0, len(res.context["object_list"]))

        # search query provided by view's get_search_query method
        res = self.client.get(
            "/searchable/predefined_query/", data={"q": "idoesntmatter"}
        )
        self.assertEqual(res.status_code, 200)
        self.assertEqual(1, len(res.context["object_list"]))

        # exact search query
        res = self.client.get("/searchable/exact_query/", data={"q": "test"})
        self.assertEqual(res.status_code, 200)
        self.assertEqual(1, len(res.context["object_list"]))

        # search query consists only of spaces
        res = self.client.get("/searchable/", data={"q": "  "})
        self.assertEqual(res.status_code, 200)
        self.assertEqual(3, len(res.context["object_list"]))

        # wrong lookup
        try:
            self.assertRaises(
                self.client.get("/searchable/wrong_lookup/", data={"q": "test"})
            )
            error = False
        except ValueError:
            error = True
        self.assertTrue(error)


class SortableViewTest(TestCase):
    def setUp(self):
        order = Order(name="Dummy Order")
        order.save()
        Item.objects.create(sku="1A", name="test A", order=order, price=0)
        Item.objects.create(sku="1B", name="test B", order=order, price=0)

    def test_sort(self):
        res = self.client.get("/sortable/fields/")
        self.assertEqual(res.status_code, 200)
        self.assertFalse(res.context["sort_helper"].is_sorted_by_name())

        asc_url = res.context["sort_helper"].get_sort_query_by_name_asc()
        res = self.client.get("/sortable/fields/%s" % asc_url)
        self.assertEqual(res.context["object_list"][0].name, "test A")
        self.assertEqual(res.context["object_list"][1].name, "test B")
        self.assertTrue(res.context["sort_helper"].is_sorted_by_name())

        desc_url = res.context["sort_helper"].get_sort_query_by_name_desc()
        res = self.client.get("/sortable/fields/%s" % desc_url)
        self.assertEqual(res.context["object_list"][0].name, "test B")
        self.assertEqual(res.context["object_list"][1].name, "test A")
        self.assertTrue(res.context["sort_helper"].is_sorted_by_name())
        # reversed sorting
        sort_url = res.context["sort_helper"].get_sort_query_by_name()
        res = self.client.get("/sortable/fields/%s" % sort_url)
        self.assertEqual(res.context["object_list"][0].name, "test A")
        sort_url = res.context["sort_helper"].get_sort_query_by_name()
        res = self.client.get("/sortable/fields/%s" % sort_url)
        self.assertEqual(res.context["object_list"][0].name, "test B")
        # can't use fields and aliases in same time
        self.assertRaises(
            ImproperlyConfigured,
            lambda: self.client.get("/sortable/fields_and_aliases/"),
        )
        # check that aliases included in params
        res = self.client.get("/sortable/aliases/")
        self.assertIn(
            "o=by_name", res.context["sort_helper"].get_sort_query_by_by_name()
        )
        self.assertIn("o=by_sku", res.context["sort_helper"].get_sort_query_by_by_sku())



from __future__ import unicode_literals

import datetime
import functools
import operator

import six
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured
from django.db.models import Q
from django.views.generic.base import ContextMixin
from six.moves import reduce

VALID_STRING_LOOKUPS = (
    "iexact",
    "contains",
    "icontains",
    "startswith",
    "istartswith",
    "endswith",
    "iendswith",
    "search",
    "regex",
    "iregex",
)


class SearchableListMixin(object):
    """
    Filter queryset like a django admin search_fields does, but with little
     more intelligence:
    if self.search_split is set to True (by default) it will split query
     to words (by whitespace)
    Also tries to convert each word to date with self.search_date_formats and
     then search each word in separate field
    e.g. with query 'foo bar' you can find object with
     obj.field1__icontains='foo' and obj.field2__icontains=='bar'

    To provide custom lookup just set one of the search_fields to tuple,
    e.g. search_fields = [('field1', 'iexact'), 'field2', ('field3', 'startswith')]

    This class is designed to be used with django.generic.ListView

    You could specify query by overriding get_search_query method
    by default this method will try to get 'q' key from request.GET
    (this can be disabled with search_use_q=False)
    """

    search_fields = ["id"]
    search_date_fields = None
    search_date_formats = ["%d.%m.%y", "%d.%m.%Y"]
    search_split = True
    search_use_q = True
    check_lookups = True

    def get_words(self, query):
        if self.search_split:
            return query.split()
        return [query]

    def get_search_fields_with_filters(self):
        fields = []
        for sf in self.search_fields:
            if isinstance(sf, six.string_types):
                fields.append((sf, "icontains"))
            else:
                if self.check_lookups and sf[1] not in VALID_STRING_LOOKUPS:
                    raise ValueError("Invalid string lookup - %s" % sf[1])
                fields.append(sf)
        return fields

    def try_convert_to_date(self, word):
        """
        Tries to convert word to date(datetime) using search_date_formats
        Return None if word fits no one format
        """
        for frm in self.search_date_formats:
            try:
                return datetime.datetime.strptime(word, frm).date()
            except ValueError:
                pass
        return None

    def get_search_query(self):
        """
        Get query from request.GET 'q' parameter when search_use_q is set to True
        Override this method to provide your own query to search
        """
        return self.search_use_q and self.request.GET.get("q", "").strip()

    def get_queryset(self):
        qs = super(SearchableListMixin, self).get_queryset()
        query = self.get_search_query()
        if query:
            w_qs = []
            search_pairs = self.get_search_fields_with_filters()
            for word in self.get_words(query):
                filters = [
                    Q(**{"%s__%s" % (pair[0], pair[1]): word}) for pair in search_pairs
                ]
                if self.search_date_fields:
                    dt = self.try_convert_to_date(word)
                    if dt:
                        filters.extend(
                            [
                                Q(**{field_name: dt})
                                for field_name in self.search_date_fields
                            ]
                        )
                w_qs.append(reduce(operator.or_, filters))
            qs = qs.filter(reduce(operator.and_, w_qs))
            qs = qs.distinct()
        return qs


class SortHelper(object):
    def __init__(
        self, request, sort_fields_aliases, sort_param_name, sort_type_param_name
    ):
        # Create a list from sort_fields_aliases, in case it is a generator,
        # since we want to iterate through it multiple times.
        sort_fields_aliases = list(sort_fields_aliases)

        self.initial_params = request.GET.copy()
        self.sort_fields = dict(sort_fields_aliases)
        self.inv_sort_fields = dict((v, k) for k, v in sort_fields_aliases)
        self.initial_sort = self.inv_sort_fields.get(
            self.initial_params.get(sort_param_name), None
        )
        self.initial_sort_type = self.initial_params.get(sort_type_param_name, "asc")
        self.sort_param_name = sort_param_name
        self.sort_type_param_name = sort_type_param_name

        for field, alias in self.sort_fields.items():
            setattr(
                self,
                "get_sort_query_by_%s" % alias,
                functools.partial(self.get_params_for_field, field),
            )
            setattr(
                self,
                "get_sort_query_by_%s_asc" % alias,
                functools.partial(self.get_params_for_field, field, "asc"),
            )
            setattr(
                self,
                "get_sort_query_by_%s_desc" % alias,
                functools.partial(self.get_params_for_field, field, "desc"),
            )
            setattr(
                self,
                "is_sorted_by_%s" % alias,
                functools.partial(self.is_sorted_by, field),
            )

    def is_sorted_by(self, field_name):
        return field_name == self.initial_sort and self.initial_sort_type or False

    def get_params_for_field(self, field_name, sort_type=None):
        """
        If sort_type is None - inverse current sort for field, if no sorted - use asc
        """
        if not sort_type:
            if self.initial_sort == field_name:
                sort_type = "desc" if self.initial_sort_type == "asc" else "asc"
            else:
                sort_type = "asc"
        self.initial_params[self.sort_param_name] = self.sort_fields[field_name]
        self.initial_params[self.sort_type_param_name] = sort_type
        return "?%s" % self.initial_params.urlencode()

    def get_sort(self):
        if not self.initial_sort:
            return None
        sort = "%s" % self.initial_sort
        if self.initial_sort_type == "desc":
            sort = "-%s" % sort
        return sort


class SortableListMixin(ContextMixin):
    """
    You can provide either sort_fields as a plain list like
     ['id', 'some', 'foo__bar', ...]
    or, if you want to hide original field names you can provide list of tuples with
    alias that will be used:
    [('id', 'by_id'), ('some', 'show_this'), ('foo__bar', 'bar')]

    If sort_param_name exists in query but sort_type_param_name is omitted queryset
     will be sorted as 'asc'
    """

    sort_fields = []
    sort_fields_aliases = []
    sort_param_name = "o"
    sort_type_param_name = "ot"

    def get_sort_fields(self):
        if self.sort_fields:
            return zip(self.sort_fields, self.sort_fields)
        return self.sort_fields_aliases

    def get_sort_helper(self):
        return SortHelper(
            self.request,
            self.get_sort_fields(),
            self.sort_param_name,
            self.sort_type_param_name,
        )

    def _sort_queryset(self, queryset):
        self.sort_helper = self.get_sort_helper()
        sort = self.sort_helper.get_sort()
        if sort:
            queryset = queryset.order_by(sort)
        return queryset

    def get_queryset(self):
        qs = super(SortableListMixin, self).get_queryset()
        if self.sort_fields and self.sort_fields_aliases:
            raise ImproperlyConfigured(
                "You should provide sort_fields or sort_fields_aliaces but not both"
            )
        return self._sort_queryset(qs)

    def get_context_data(self, **kwargs):
        context = {}
        if hasattr(self, "sort_helper"):
            context["sort_helper"] = self.sort_helper
        context.update(kwargs)
        return super(SortableListMixin, self).get_context_data(**context)


class SuccessMessageWithInlinesMixin(object):
    """
    Adds a success message on successful form submission.
    """

    success_message = ""

    def forms_valid(self, form, inlines):
        response = super(SuccessMessageWithInlinesMixin, self).forms_valid(
            form, inlines
        )
        success_message = self.get_success_message(form.cleaned_data)
        if success_message:
            messages.success(self.request, success_message)
        return response

    def get_success_message(self, cleaned_data):
        return self.success_message % cleaned_data



from extra_views.advanced import (
    CreateWithInlinesView,
    FormSetSuccessMessageMixin,
    InlineFormSet,
    InlineFormSetFactory,
    NamedFormsetsMixin,
    SuccessMessageMixin,
    UpdateWithInlinesView,
)
from extra_views.contrib.mixins import SearchableListMixin, SortableListMixin
from extra_views.dates import CalendarMonthView
from extra_views.formsets import (
    BaseFormSetMixin,
    BaseInlineFormSetMixin,
    FormSetView,
    InlineFormSetView,
    ModelFormSetView,
)

__version__ = "0.13.0"

__all__ = [
    "CreateWithInlinesView",
    "FormSetSuccessMessageMixin",
    "InlineFormSet",
    "InlineFormSetFactory",
    "NamedFormsetsMixin",
    "SuccessMessageMixin",
    "UpdateWithInlinesView",
    "SearchableListMixin",
    "SortableListMixin",
    "CalendarMonthView",
    "BaseFormSetMixin",
    "BaseInlineFormSetMixin",
    "FormSetView",
    "InlineFormSetView",
    "ModelFormSetView",
]



import re

from setuptools import setup

# get version without importing
with open("extra_views/__init__.py", "rb") as f:
    VERSION = str(re.search('__version__ = "(.+?)"', f.read().decode("utf-8")).group(1))

setup(
    name="django-extra-views",
    version=VERSION,
    url="https://github.com/AndrewIngram/django-extra-views",
    install_requires=["Django >=1.11", "six>=1.5.2"],
    description="Extra class-based views for Django",
    long_description=open("README.rst", "r").read(),
    license="MIT",
    author="Andrew Ingram",
    author_email="andy@andrewingram.net",
    packages=["extra_views", "extra_views.contrib"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],
)
